DROP VIEW IF EXISTS spermission_v;
DROP VIEW IF EXISTS risk_level_v;
DROP VIEW IF EXISTS current_user_access_v;
DROP VIEW IF EXISTS suser_brole_v;
DROP VIEW IF EXISTS deleted_reviewers_v;
DROP VIEW IF EXISTS admin_auth_assignments_v;
DROP VIEW IF EXISTS sgroup_indirect_suser_v;
DROP VIEW IF EXISTS sgroup_path_v;
DROP VIEW IF EXISTS sgroups_subgroups_v;
DROP VIEW IF EXISTS suser_sod_case_info_v;
DROP VIEW IF EXISTS sod_case_info_v;
DROP VIEW IF EXISTS role_perm_policy_v;
DROP VIEW IF EXISTS sod_policy_all_v;
DROP VIEW IF EXISTS sod_policy_v;
DROP VIEW IF EXISTS policy_owners_v;
DROP VIEW IF EXISTS sod_case_violation_v;
DROP VIEW IF EXISTS sod_policy_violation_v;
DROP VIEW IF EXISTS spermission_associations_v;
DROP VIEW IF EXISTS spermission_suser_v;
DROP VIEW IF EXISTS business_role_nested_v;

CREATE VIEW business_role_nested_v AS 
  WITH RECURSIVE nested_broles(parent_id, child_id, depth, path) AS (
    SELECT 
      base.parent_id,
      base.child_id,
      1,
      base.parent_id::text AS parent_id
    FROM ( SELECT 
        incl_brole.business_role_id AS parent_id,
        child.id AS child_id
      FROM business_role_incl_brole incl_brole
        JOIN business_role parent ON parent.id = incl_brole.business_role_id AND parent.approval_state = 'PUBLISHED'
        JOIN business_role child ON child.unique_id = incl_brole.unique_brole_id AND child.approval_state = 'PUBLISHED') base
    
    UNION ALL
    
    SELECT 
      base.parent_id,
      base.child_id,
      nb.depth + 1,
      nb.path || ', ' || base.parent_id
    FROM ( SELECT 
        incl_brole.business_role_id AS parent_id,
        child.id AS child_id
      FROM business_role_incl_brole incl_brole
        JOIN business_role parent ON parent.id = incl_brole.business_role_id AND parent.approval_state = 'PUBLISHED'
        JOIN business_role child ON child.unique_id = incl_brole.unique_brole_id AND child.approval_state = 'PUBLISHED') base
        JOIN nested_broles nb ON base.parent_id = nb.child_id
  )
  SELECT 
    nested_broles.parent_id,
    nested_broles.child_id,
    nested_broles.depth,
    nested_broles.path
  FROM nested_broles;

CREATE VIEW spermission_suser_v AS
SELECT
    sap.suser,
    perm.id permission_id,
    perm.name permission_name,
    perm.description permission_description,
    perm.type permission_type,
    data_source.name application_name,
    perm.risk permission_risk
FROM
    (SELECT id, name, description, type, risk, application FROM spermission WHERE (exclude_from_catalog = false OR exclude_from_catalog IS NULL) AND effective = true) perm
    JOIN data_source ON perm.application = data_source.id
    JOIN (SELECT DISTINCT suser, spermission FROM suser_account_permission) sap ON perm.id = sap.spermission;

CREATE VIEW spermission_associations_v AS
 SELECT perm.id,
    perm.name permission,
    perm.type permission_type,
    perm.exclude_from_catalog,
    perm_assoc.name associated_permission,
    perm_assoc.type associated_permission_type,
    perm_entail.entailed associated_permission_id,
    perm_entail.distance
   FROM ((spermission_entailment perm_entail
     LEFT JOIN spermission perm ON ((perm.collected_entity = perm_entail.entailing)))
     LEFT JOIN spermission perm_assoc ON ((perm_assoc.collected_entity = perm_entail.entailed)))
  WHERE ((((perm.deleted = false) AND (perm.effective = true)) AND (perm_assoc.deleted = false)) AND (perm_assoc.effective = true));

CREATE VIEW sod_policy_violation_v AS 
 SELECT sod_case_info.id AS case_id,
    sod_case_info.state,
    sod_case_info.current_violation,
    sod_case_info.policy AS policy_id,
    sod_case_info.policy_name,
    sod_case_info.violator_name,
    sod_case_info.violator_title,
    sod_violation.suser,
    sod_violation.saccount,
    sod_condition.id AS condition_id,
    sod_condition.type AS cond_type,
    sod_condition.cond_order,
    sod_condition_item.unique_id,
        CASE
            WHEN sod_condition_item.type::text = 'PERMISSION'::text THEN spermission.name
            ELSE role_policy.name
        END AS item_name,
    sod_condition_item.type AS item_type,
    sod_condition_item.item_order
   FROM sod_case_info
     LEFT JOIN sod_condition ON sod_condition.policy = sod_case_info.policy
     LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
     LEFT JOIN sod_violation ON sod_violation.id = sod_case_info.current_violation
     LEFT JOIN spermission ON spermission.unique_permission_id::text = sod_condition_item.unique_id::text AND spermission.effective = true AND spermission.deleted = false
     LEFT JOIN role_policy ON role_policy.unique_policy_id::text = sod_condition_item.unique_id::text AND role_policy.deleted = false
  WHERE (sod_condition_item.unique_id::text IN ( SELECT DISTINCT spermission_1.unique_permission_id
           FROM saccount_user
             JOIN saccount_permission ON saccount_permission.saccount = saccount_user.saccount
             JOIN spermission spermission_1 ON spermission_1.id = saccount_permission.spermission
          WHERE saccount_user.saccount = sod_violation.saccount OR saccount_user.suser = sod_violation.suser)) OR (sod_condition_item.unique_id::text IN ( SELECT DISTINCT role_policy_1.unique_policy_id
           FROM suser
             JOIN role_user ON role_user.suser = suser.id
             JOIN role_policy role_policy_1 ON role_policy_1.id = role_user.policy
          WHERE suser.id = sod_violation.suser));

CREATE VIEW sod_case_violation_v AS 
SELECT DISTINCT users.id,
    spermission.id AS perm_id,
    users.unique_user_id,
    users.display_name,
    spv.unique_id,
    spv.item_name,
    string_agg(concat(spv.policy_name, concat(': SOD-', lpad(spv.case_id::character varying::text, 4, '0'::text))), ', '::text ORDER BY users.display_name) AS policy_name
   FROM suser users
     LEFT JOIN ( SELECT sc.id AS case_id,
                CASE
                    WHEN sv.suser IS NOT NULL THEN su.display_name
                    ELSE sa.name
                END AS violator_name,
                CASE
                    WHEN sod_condition_item.type::text = 'PERMISSION'::text THEN spermission_1.name
                    ELSE role_policy.name
                END AS item_name,
            sp.name AS policy_name,
            sod_condition_item.unique_id
           FROM sod_case sc
             LEFT JOIN sod_policy sp ON sc.policy = sp.id
             LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
             LEFT JOIN suser su ON sv.suser = su.id
             LEFT JOIN saccount sa ON sv.saccount = sa.id
             LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
             LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
             LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id::text = sod_condition_item.unique_id::text AND spermission_1.effective = true AND spermission_1.deleted = false
             LEFT JOIN role_policy ON role_policy.unique_policy_id::text = sod_condition_item.unique_id::text AND role_policy.deleted = false) spv ON spv.violator_name::text = users.display_name::text
     LEFT JOIN ( SELECT spermission_1.id,
            spermission_1.unique_permission_id
           FROM spermission spermission_1
          WHERE (spermission_1.exclude_from_catalog = false OR spermission_1.exclude_from_catalog IS NULL) AND spermission_1.effective = true) spermission ON spermission.unique_permission_id::text = spv.unique_id::text
  WHERE spv.policy_name IS NOT NULL AND users.effective = true
  GROUP BY users.id, spv.unique_id, spv.item_name, spermission.id;


CREATE VIEW policy_owners_v AS
 SELECT policy_owner.is_user,
    policy_owner.unique_owner_id,
    policy_owner.role_policy,
    policy_owner.sod_policy,
        CASE
            WHEN (policy_owner.is_user = true) THEN suser.display_name
            WHEN (policy_owner.is_user = false) THEN sgroup.name
            ELSE NULL
        END owner_name
   FROM ((policy_owner
     LEFT JOIN suser ON ((((suser.unique_user_id) = (policy_owner.unique_owner_id)) AND (suser.effective = true))))
     LEFT JOIN sgroup ON ((((sgroup.unique_group_id) = (policy_owner.unique_owner_id)) AND (sgroup.effective = true))));


CREATE VIEW sod_policy_v AS
 SELECT sp.id policy_id,
    sp.deleted policy_status,
    change.display_name changed_by,
    creater.display_name created_by,
    sp.creation_time,
    sp.description policy_description,
    sp.detection_status,
    sp.name policy_name,
    sp.risk,
    sp.state,
    sp.unique_policy_id,
    sp.update_time,
    sp.control_period,
    sp.resolve,
    policy_owners.owners
   FROM (((sod_policy sp
     LEFT JOIN suser change ON (((((change.unique_user_id) = (sp.changed_by)) AND (change.deleted = false)) AND (change.effective = true))))
     LEFT JOIN suser creater ON (((((creater.unique_user_id) = (sp.created_by)) AND (creater.deleted = false)) AND (creater.effective = true))))
     LEFT JOIN ( SELECT policy_owners_v.sod_policy,
            string_agg((policy_owners_v.owner_name), ', ') owners
           FROM policy_owners_v
          GROUP BY policy_owners_v.sod_policy) policy_owners ON ((policy_owners.sod_policy = sp.id)))
  WHERE (sp.deleted = false);

CREATE VIEW sod_policy_all_v AS 
  SELECT 
    sp.id AS policy_id,
    sp.deleted AS policy_status,
    change.display_name AS changed_by,
    creater.display_name AS created_by,
    sp.creation_time,
    sp.description AS policy_description,
    sp.detection_status,
    sp.name AS policy_name,
    sp.risk,
    sp.state,
    sp.unique_policy_id,
    sp.update_time,
    sp.control_period,
    sp.resolve,
    policy_owners.owners
  FROM sod_policy sp
    LEFT JOIN suser change ON change.unique_user_id = sp.changed_by AND change.deleted = false AND change.effective = true
    LEFT JOIN suser creater ON creater.unique_user_id = sp.created_by AND creater.deleted = false AND creater.effective = true
    LEFT JOIN (SELECT 
        policy_owners_v.sod_policy,
        string_agg(policy_owners_v.owner_name, ' | ') AS owners
      FROM policy_owners_v
      GROUP BY policy_owners_v.sod_policy) policy_owners ON policy_owners.sod_policy = sp.id;


CREATE VIEW role_perm_policy_v AS 
SELECT DISTINCT 
  policy.id,
  policy.name AS policy_name,
  policy.unique_policy_id,
  policy.description AS policy_description,
  policy.created_by,
  policy.changed_by,
  policy.creation_time,
  policy.update_time,
  policy.deleted,
  CASE
    WHEN policy.state = 'INACTIVE' AND policy.detection_status IS NULL OR policy.detection_status = 'INACTIVE' THEN 'Inactive'
    WHEN policy.state = 'ACTIVE' AND policy.detection_status = 'COMPLETED' OR policy.detection_status = 'RUNNING' THEN 'Active'
    WHEN policy.state = 'INACTIVE' AND policy.detection_status = 'INVALID' THEN 'Invalid from inactive'
    WHEN policy.state = 'INVALID' AND policy.detection_status = 'INVALID' THEN 'Invalid from active'
    ELSE NULL
  END AS current_state,
  policy.state AS actual_state,
  policy.detection_status,
  policy.risk_mode AS policy_risk_mode,
  policy.risk AS policy_risk,
  policy.cost AS policy_cost,
  ( SELECT DISTINCT string_agg(subquery.unique_owner_id, ' | ') AS user_owner_id
    FROM ( SELECT DISTINCT powner.unique_owner_id
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = true AND policy.id = rperm.id) subquery) AS user_owner_id,
  ( SELECT DISTINCT string_agg(subquery2.unique_owner_id, ' | ') AS group_owner_id
    FROM ( SELECT DISTINCT powner.unique_owner_id
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = false AND policy.id = rperm.id) subquery2) AS group_owner_id,
  ( SELECT DISTINCT string_agg(subquery.owner_name, ' | ') AS user_owners
    FROM ( SELECT DISTINCT powner.owner_name
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = true AND policy.id = rperm.id) subquery) AS user_owners,
  ( SELECT DISTINCT string_agg(subquery2.owner_name, ' | ') AS group_owners
    FROM ( SELECT DISTINCT powner.owner_name
      FROM policy_owners_v powner
      LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
      WHERE powner.is_user = false AND policy.id = rperm.id) subquery2) AS group_owners,
  perm.unique_permission_id,
  perm.name AS perm_name,
  perm_user.unique_user_id AS permission_user_id,
  perm_user.display_name AS permission_user_name
FROM 
  role_policy policy
  LEFT JOIN policy_owner ON policy.id = policy_owner.role_policy
  LEFT JOIN role_permission role_perm ON policy.id = role_perm.policy
  LEFT JOIN spermission perm ON role_perm.unique_permission_id = perm.unique_permission_id AND perm.effective = true
  LEFT JOIN suser_account_permission_role role_user ON role_user.role = policy.id
  LEFT JOIN suser perm_user ON perm_user.id = role_user.suser;


CREATE VIEW public.sod_case_info_v AS 
 SELECT sc.id,
        CASE
            WHEN sc.state::text = 'APPROVED'::text AND sc.approval_expire_time IS NOT NULL AND sc.approval_expire_time < now() THEN 'APPROVAL_EXPIRED'::character varying
            ELSE sc.state
        END AS state,
    sc.approval_expire_time,
    sc.current_violation,
    sc.policy,
    sv.suser IS NOT NULL AS violator_is_user,
        CASE
            WHEN sv.suser IS NOT NULL THEN su.display_name
            ELSE sa.name
        END AS violator_name,
        CASE
            WHEN sv.suser IS NOT NULL THEN su.unique_user_id
            ELSE sa.unique_account_id
        END AS violator_id,
        CASE
            WHEN sv.suser IS NOT NULL THEN su.job_title
            ELSE NULL::character varying
        END AS violator_title,
    sp.name AS policy_name
   FROM sod_case sc
     LEFT JOIN sod_policy sp ON sc.policy = sp.id
     LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
     LEFT JOIN suser su ON sv.suser = su.id
     LEFT JOIN saccount sa ON sv.saccount = sa.id;


CREATE VIEW suser_sod_case_info_v AS
SELECT 
    suser.id,
    suser.collected_entity,
    suser.display_name,
    suser.job_title,
    suser_email.email,
    suser.department,
    suser.user_id,
    suser.first_name,
    suser.last_name,
    suser.risk,
    suser.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations
FROM suser
     LEFT JOIN suser supervisor ON supervisor.unique_user_id = suser.supervisor_unique_id AND supervisor.effective = true AND supervisor.deleted = false
     LEFT JOIN ( SELECT 
                     suser1.id AS suser_id,
                     suser1.display_name,
                     string_agg(suser_email1.email, ' | ' ORDER BY (suser_email1.email)) AS email
                 FROM suser suser1
                     JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = true AND suser1.deleted = false
                 GROUP BY suser1.id, suser1.display_name) suser_email ON suser.id = suser_email.suser_id
     LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     string_agg(concat(sci1.policy_name, concat(': SOD-', lpad(UPPER(to_hex(sci1.id))::character varying::text, 4, '0'::text))),
                                ', ' ORDER BY sci1.violator_name) AS violations
                 FROM sod_case_info_v sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = suser.unique_user_id
WHERE suser.effective = true AND suser.deleted = false;

CREATE VIEW sgroups_subgroups_v AS 
 SELECT sub.supergroups,
    parent.name AS parent_name,
    sub.subgroups,
    child.name AS child_name
   FROM sgroup_subgroups sub
     LEFT JOIN sgroup parent ON parent.id = sub.supergroups
     LEFT JOIN sgroup child ON child.id = sub.subgroups
  WHERE parent.effective = true AND parent.deleted = false AND child.effective = true AND child.deleted = false;

CREATE VIEW sgroup_path_v AS 
 WITH RECURSIVE search_graph(id, parent_name, subgroups, child_name, depth, path, group_path, cycle) AS (
         SELECT g.supergroups,
            g.parent_name,
            g.subgroups,
            g.child_name,
            1,
            g.supergroups::text AS supergroups,
            g.parent_name::text || '\'::text,
            false AS bool
           FROM sgroups_subgroups_v g
        UNION ALL
         SELECT g.supergroups,
            g.parent_name,
            g.subgroups,
            g.child_name,
            sg.depth + 1,
            (sg.path || ', '::text) || g.supergroups::text,
            (sg.group_path || g.parent_name::text) || '\'::text,
            sg.path ~~ (('%'::text || g.supergroups::text) || '%'::text)
           FROM sgroups_subgroups_v g,
            search_graph sg
          WHERE g.supergroups = sg.subgroups AND NOT sg.cycle
        )
 SELECT search_graph.id,
    search_graph.parent_name,
    search_graph.subgroups,
    search_graph.child_name,
    search_graph.depth,
    search_graph.path,
    search_graph.group_path,
    search_graph.cycle
   FROM search_graph;

CREATE VIEW sgroup_indirect_suser_v AS 
 SELECT groups.parent_id,
    parent.name AS parent_name,
    groups.subgroups AS child_id,
    child.name AS child_name,
    suser.id,
    suser.unique_user_id,
    suser.display_name,
    suser_email.email,
    suser.department,
    suser.job_title
   FROM ( SELECT to_number(unnest(string_to_array(sgroup_path_v.path, ', '::text)), '9999'::text) AS parent_id,
            sgroup_path_v.subgroups
           FROM sgroup_path_v
        UNION
         SELECT sgroup_path_v.id AS parent_id,
            sgroup_path_v.subgroups
           FROM sgroup_path_v) groups
     JOIN sgroup parent ON groups.parent_id = parent.id::numeric AND parent.effective = true AND parent.deleted = false
     JOIN sgroup child ON groups.subgroups = child.id AND child.effective = true AND child.deleted = false
     JOIN ( SELECT sgroup_user.suser AS suser_id,
            sgroup.id
           FROM sgroup_user
             JOIN sgroup ON sgroup_user.sgroup = sgroup.id AND sgroup.deleted = false AND sgroup.effective = true) sgroup_suser ON groups.subgroups = sgroup_suser.id
     JOIN suser ON sgroup_suser.suser_id = suser.collected_entity AND suser.effective = true
     LEFT JOIN ( SELECT suser_1.id AS suser,
            suser_1.display_name,
            string_agg(suser_email_1.email::text, ' | '::text ORDER BY (suser_email_1.email::text)) AS email
           FROM suser suser_1
             JOIN suser_email suser_email_1 ON suser_1.id = suser_email_1.suser AND suser_1.effective = true AND suser_1.deleted = false
          GROUP BY suser_1.id, suser_1.display_name) suser_email ON suser.id = suser_email.suser;

CREATE VIEW admin_auth_assignments_v AS
 SELECT mappings.role_display_name AS role,
        COALESCE(direct.display_name, direct_group_user.display_name) AS user_display_name,
        COALESCE(direct_group_user.unique_user_id, mappings.unique_user_id) AS unique_user_id,
        mappings.unique_group_id,
        sgroup.name AS group_name,
        CASE 
            WHEN direct.id IS NOT NULL THEN TRUE
            WHEN sgroup_user.suser IS NOT NULL THEN FALSE
            ELSE NULL
        END AS direct_user

FROM   
(SELECT DISTINCT
                ar.role_display_name,
                arm.unique_user_id,
                arm.unique_group_id
FROM auth_role ar
JOIN auth_role_mapping arm ON ar.id = arm.auth_role 
WHERE ar.role_type = 0 AND NOT (arm.unique_user_id IS NULL AND arm.unique_group_id IS NULL)) mappings
LEFT JOIN suser direct ON direct.unique_user_id = mappings.unique_user_id AND direct.effective = TRUE
LEFT JOIN sgroup ON sgroup.unique_group_id = mappings.unique_group_id AND sgroup.effective = TRUE
LEFT JOIN sgroup_user ON sgroup_user.sgroup = sgroup.id
LEFT JOIN suser direct_group_user ON direct_group_user.collected_entity = sgroup_user.suser AND direct_group_user.effective = TRUE
LEFT JOIN sgroup_indirect_suser_v indirect ON indirect.parent_id = sgroup.id
WHERE sgroup_user.sversion = (SELECT MAX(sversion) FROM sgroup_user) AND sgroup_user.suser IS NOT NULL OR indirect.display_name IS NOT NULL OR direct.display_name IS NOT NULL
ORDER BY user_display_name;

CREATE VIEW deleted_reviewers_v AS 
   WITH all_reviewers AS (
      SELECT 
        item.id item_id,
        single_reviewer.display_name reviewer,
        single_reviewer.unique_user_id,
        single_reviewer.deleted,
        single_reviewer.effective,
        single_reviewer.timestamp time_deletion,
        defn.name review_name,
        instance.id review_instance_id,
        instance.status,
        item.item_type,
        item_user.display_name AS user_display_name,
        CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.name
          WHEN item.item_type = 'ACCOUNT' THEN acct.name
          WHEN item.item_type = 'ROLE' THEN role.name
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
        END review_item
      FROM review_item item
        JOIN (SELECT 
            action.review_item_id item_id,
            suser.display_name,
            suser.unique_user_id,
            suser.deleted,
            suser.effective,
            COALESCE(suser.last_modified, (SELECT min(timestamp) FROM sversion JOIN data_producer dp ON sversion.data_producer = dp.id WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)) AS timestamp
          FROM review_item_action action
            JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = false
            JOIN suser ON ri.single_reviewer_unique_id = suser.unique_user_id AND suser.effective = true
          WHERE action.review_item_status <> 'OVERRIDDEN' AND action.action <> 'REASSIGNED') single_reviewer ON item.id = single_reviewer.item_id
        JOIN suser item_user ON item_user.unique_user_id = item.unique_user_id AND item_user.effective = true
        JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status IN ('RUNNING','PREVIEW')
        JOIN (SELECT
            review_instance.id inst_id,
            CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name
          FROM review_definition defn
            JOIN review_instance ON defn.id = review_instance.review_definition
            LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
        LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = true
        LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = true
        LEFT JOIN business_role brole ON brole.id = item.business_role_id
        LEFT JOIN role_policy role ON role.id = item.role_id
    UNION
      SELECT 
        item.id AS item_id,
        multi_reviewer.display_name AS reviewer,
        multi_reviewer.unique_user_id,
        multi_reviewer.deleted,
        multi_reviewer.effective,
        multi_reviewer.timestamp AS time_deletion,
        defn.name AS review_name,
        instance.id AS review_instance_id,
        instance.status,
        item.item_type,
        item_user.display_name AS user_display_name,
        CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.name
          WHEN item.item_type = 'ACCOUNT' THEN acct.name
          WHEN item.item_type = 'ROLE' THEN role.name
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
        END AS review_item
      FROM review_item item
        JOIN (SELECT 
              action.review_item_id AS item_id,
              action.review_instance_id,
              suser.display_name,
              suser.unique_user_id,
              suser.deleted,
              suser.effective,
              COALESCE(suser.last_modified,(SELECT min(timestamp) FROM sversion JOIN data_producer dp ON sversion.data_producer = dp.id WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)) AS timestamp
            FROM review_item_action action
              JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = true
              JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
              JOIN (SELECT 
                  suser.display_name,
                  suser.unique_user_id,
                  suser.deleted,
                  suser.effective,
                  suser.sversion,
                  suser.last_modified
                FROM suser
                WHERE suser.effective = true) suser ON rsi.unique_user_id = suser.unique_user_id
            WHERE action.action <> 'REASSIGNED'
          UNION
            SELECT 
              action.review_item_id AS item_id,
              action.review_instance_id,
              sgroup_users.display_name,
              sgroup_users.unique_user_id,
              sgroup_users.deleted,
              sgroup_users.effective,
              COALESCE(sgroup_users.last_modified, (SELECT min(timestamp) FROM sversion JOIN data_producer dp ON sversion.data_producer = dp.id WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sgroup_users.sversion)) AS timestamp
            FROM review_item_action action
              JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = true
              JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
              JOIN (SELECT 
                    sgroup.unique_group_id,
                    suser.display_name,
                    suser.unique_user_id,
                    suser.deleted,
                    suser.effective,
                    suser.sversion,
                    suser.last_modified
                  FROM sgroup
                    JOIN sgroup_user gu ON gu.sgroup = sgroup.id
                    JOIN suser ON gu.suser = suser.collected_entity AND suser.effective = true
                UNION
                  SELECT 
                    sgroup.unique_group_id,
                    suser.display_name,
                    suser.unique_user_id,
                    suser.deleted,
                    suser.effective,
                    suser.sversion,
                    suser.last_modified
                  FROM sgroup
                    JOIN sgroup_user gu ON gu.sgroup = sgroup.id
                    JOIN suser ON gu.suser = suser.collected_entity AND (SELECT MAX(sversion) FROM sgroup_user WHERE sgroup = sgroup.id) = suser.sversion AND sgroup.effective = true
                UNION 
                  SELECT 
                    sgroup.unique_group_id,
                    suser.display_name,
                    suser.unique_user_id,
                    suser.deleted,
                    suser.effective,
                    suser.sversion,
                    suser.last_modified
                  FROM sgroup
                    JOIN sgroup_indirect_suser_v indirect ON indirect.parent_id = sgroup.id
                    JOIN suser ON indirect.unique_user_id = suser.unique_user_id AND suser.effective = true) sgroup_users ON rsi.unique_group_id = sgroup_users.unique_group_id
            WHERE action.action <> 'REASSIGNED') multi_reviewer ON item.id = multi_reviewer.item_id
        JOIN suser item_user ON item_user.unique_user_id = item.unique_user_id AND item_user.effective = true
        JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status IN ('RUNNING','PREVIEW')
        JOIN (SELECT
            review_instance.id inst_id,
            CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name
          FROM review_definition defn
            JOIN review_instance ON defn.id = review_instance.review_definition
            LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
        LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = true
        LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = true
        LEFT JOIN business_role brole ON brole.id = item.business_role_id
        LEFT JOIN role_policy role ON role.id = item.role_id
   )
  SELECT 
    deleted_reviewers.item_id,
    deleted_reviewers.reviewer AS deleted_reviewer,
    deleted_reviewers.unique_user_id,
    curr_reviewers.display_name AS curr_reviewers,
    deleted_reviewers.time_deletion,
    deleted_reviewers.review_name,
    deleted_reviewers.review_instance_id,
    deleted_reviewers.status,
    deleted_reviewers.item_type,
    deleted_reviewers.review_item,
    deleted_reviewers.user_display_name
  FROM all_reviewers deleted_reviewers
    LEFT JOIN (SELECT 
        all_reviewers.item_id,
        string_agg(all_reviewers.reviewer, ' | ' ORDER BY (all_reviewers.reviewer)) AS display_name
      FROM all_reviewers
      WHERE all_reviewers.deleted = false and all_reviewers.effective = true
      GROUP BY all_reviewers.item_id) curr_reviewers ON curr_reviewers.item_id = deleted_reviewers.item_id
   WHERE deleted_reviewers.deleted = true AND deleted_reviewers.effective = true;

CREATE VIEW suser_brole_v AS 
  SELECT 
    brole.id brole_id,
    brole.name brole_name,
    suser.unique_user_id user_id,
    suser.display_name user_name,
    brole.description,
    brole.risk,
    brole.update_time
  FROM business_role brole
    JOIN (SELECT
      memb.business_role_id,
      COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
    FROM business_role_incl_memb memb
      LEFT JOIN suser ON suser.unique_user_id = memb.unique_member_id AND suser.effective = true AND suser.deleted = false
      LEFT JOIN sgroup ON sgroup.unique_group_id = memb.unique_member_id AND sgroup.effective = true
      LEFT JOIN (SELECT 
          sgroup.id,
          sgroup.unique_group_id,
          suser.unique_user_id,
          suser.display_name
        FROM sgroup 
          JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
          JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = true AND suser.deleted = false
        WHERE sgroup.effective = true) direct ON sgroup.unique_group_id = direct.unique_group_id
      LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
    WHERE NOT (memb.is_user = false AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)
    
    EXCEPT
    
    SELECT
      excl_memb.business_role_id,
      COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
    FROM business_role_excl_memb excl_memb
    LEFT JOIN suser ON suser.unique_user_id = excl_memb.unique_member_id AND suser.effective = true AND suser.deleted = false
    LEFT JOIN sgroup ON sgroup.unique_group_id = excl_memb.unique_member_id AND sgroup.effective = true
    LEFT JOIN (SELECT 
          sgroup.id,
          sgroup.unique_group_id,
          suser.unique_user_id,
          suser.display_name
        FROM sgroup 
          JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
          JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = true AND suser.deleted = false
        WHERE sgroup.effective = true) direct ON sgroup.unique_group_id = direct.unique_group_id
      LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
    WHERE NOT (excl_memb.is_user = false AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)) users ON users.business_role_id = brole.id
    JOIN suser ON users.user_id = suser.unique_user_id AND suser.effective = true
  WHERE brole.approval_state = 'PUBLISHED'

  UNION 

  SELECT 
    indirect_brole_users.parent_id brole_id, 
    indirect_brole_users.parent_name brole_name, 
    indirect_brole_users.unique_user_id user_id, 
    indirect_brole_users.display_name user_name,
    indirect_brole_users.description,
    indirect_brole_users.risk,
    indirect_brole_users.update_time
  FROM
    (SELECT
      broles.parent_id,
      parent.name parent_name,
      broles.child_id,
      child.name,
      parent.description,
      parent.update_time,
      suser.unique_user_id,
      suser.display_name,
      parent.risk
    FROM (
      SELECT 
        to_number(unnest(string_to_array(business_role_nested_v.path, ', ')), '9999') AS parent_id,
        business_role_nested_v.child_id
      FROM business_role_nested_v

      UNION

      SELECT 
        business_role_nested_v.parent_id,
        business_role_nested_v.child_id
      FROM business_role_nested_v) broles
        JOIN business_role parent ON broles.parent_id = parent.id
        JOIN business_role child ON broles.child_id = child.id
        JOIN (SELECT
            memb.business_role_id,
            COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
          FROM business_role_incl_memb memb
        LEFT JOIN suser ON suser.unique_user_id = memb.unique_member_id AND suser.effective = true AND suser.deleted = false
        LEFT JOIN sgroup ON sgroup.unique_group_id = memb.unique_member_id AND sgroup.effective = true
        LEFT JOIN (SELECT 
            sgroup.id,
            sgroup.unique_group_id,
            suser.unique_user_id,
            suser.display_name
          FROM sgroup 
            JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
            JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = true AND suser.deleted = false
          WHERE sgroup.effective = true) direct ON sgroup.unique_group_id = direct.unique_group_id
        LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
      WHERE NOT (memb.is_user = false AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)

      EXCEPT

      SELECT
        excl_memb.business_role_id,
        COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
      FROM business_role_excl_memb excl_memb
        LEFT JOIN suser ON suser.unique_user_id = excl_memb.unique_member_id AND suser.effective = true AND suser.deleted = false
        LEFT JOIN sgroup ON sgroup.unique_group_id = excl_memb.unique_member_id AND sgroup.effective = true
        LEFT JOIN (SELECT 
            sgroup.id,
            sgroup.unique_group_id,
            suser.unique_user_id,
            suser.display_name
          FROM sgroup 
            JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
            JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = true AND suser.deleted = false
          WHERE sgroup.effective = true) direct ON sgroup.unique_group_id = direct.unique_group_id
        LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
      WHERE NOT (excl_memb.is_user = false AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)) groups_users ON child.id = groups_users.business_role_id
    JOIN suser ON groups_users.user_id = suser.unique_user_id AND suser.effective = true AND suser.deleted = false) indirect_brole_users;

CREATE VIEW current_user_access_v AS
 WITH current_user_access AS (SELECT DISTINCT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
      CASE
        WHEN sgroup.group_id IS NOT NULL THEN '1-GROUP'
        ELSE '1-NO-GROUP'
      END AS detail_type,
      sgroup.group_id AS group_id,
      sgroup.name AS group_name,
      sgroup.short_description AS group_description,
      NULL::bigint AS permission_id,
      NULL AS permission_name,
      NULL AS permission_description,
      NULL AS permission_type,
      NULL AS permission_application_name,
      NULL::bigint AS permission_risk,
      NULL AS permission_root,
      NULL AS permission_policy_violated,
      NULL AS permission_entitlements,
      NULL::bigint AS brole_id,
      NULL AS brole_name,
      NULL AS brole_description,
      NULL::bigint AS brole_risk,
      NULL::timestamp with time zone AS brole_update_time,
      NULL AS supervisee_id,
      NULL AS supervisee_display_name,
      NULL AS supervisee_email,
      NULL AS supervisee_department,
      NULL AS supervisee_job_title,
      NULL::bigint AS trole_policy_id,
      NULL AS trole_item_name,
      NULL AS trole_unique_policy_id,
      NULL AS trole_description,
      NULL::bigint AS trole_risk,
      NULL AS trole_policy_violated,
      NULL AS account_name,
      NULL AS account_id,
      NULL::bigint AS account_risk,
      NULL AS account_description,
      NULL AS account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN sgroup_user direct ON direct.suser = users.collected_entity
      LEFT JOIN sgroup_indirect_suser_v indirect ON users.id = indirect.id AND direct.sgroup = indirect.child_id
      LEFT JOIN sgroup ON sgroup.id = direct.sgroup OR sgroup.id = indirect.parent_id
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN perm.permission_id IS NOT NULL THEN '2-PERMISSION'
          ELSE '2-NO-PERM'
        END AS detail_type,
      NULL::character varying AS group_id,
      NULL::character varying AS group_name,
      NULL::character varying AS group_description,
      perm.permission_id,
      perm.permission_name,
      perm.permission_description,
      perm.permission_type,
      perm.application_name AS permission_application_name,
      perm.permission_risk,
      root_perm.permission AS permission_root,
      spv.policy_name AS permission_policy_violated,
      string_agg(ent.associated_permission, ' | ' ORDER BY (ent.associated_permission)) AS permission_entitlements,
      NULL::bigint AS brole_id,
      NULL AS brole_name,
      NULL AS brole_description,
      NULL::bigint AS brole_risk,
      NULL::timestamp with time zone AS brole_update_time,
      NULL AS supervisee_id,
      NULL AS supervisee_display_name,
      NULL AS supervisee_email,
      NULL AS supervisee_department,
      NULL AS supervisee_job_title,
      NULL::bigint AS trole_policy_id,
      NULL AS trole_item_name,
      NULL AS trole_unique_policy_id,
      NULL AS trole_description,
      NULL::bigint AS trole_risk,
      NULL AS trole_policy_violated,
      NULL AS account_name,
      NULL AS account_id,
      NULL::bigint AS account_risk,
      NULL AS account_description,
      NULL AS account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN spermission_suser_v perm ON users.id = perm.suser
      LEFT JOIN (SELECT DISTINCT ent.id, ent.associated_permission FROM spermission_associations_v ent) ent ON ent.id = perm.permission_id
      LEFT JOIN spermission perm1 ON perm1.id = perm.permission_id
      LEFT JOIN spermission_associations_v root_perm ON root_perm.associated_permission_id = perm1.collected_entity
      LEFT JOIN sod_policy_violation_v spv ON spv.unique_id = perm1.unique_permission_id
    GROUP BY users.id, users.display_name, users.job_title, users.email, users.department, users.user_id, users.first_name, users.last_name, users.risk, users.unique_user_id, users.supervisor, users.violations, perm.permission_id, perm.permission_name, perm.permission_description, perm.permission_type, perm.application_name, perm.permission_risk, root_perm.permission, spv.policy_name
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN brole.brole_id IS NOT NULL THEN '3-BROLE'
          ELSE '3-NO-BROLE'
        END AS detail_type,
      NULL::character varying AS group_id,
      NULL AS group_name,
      NULL AS group_description,
      NULL::bigint AS permission_id,
      NULL AS permission_name,
      NULL AS permission_description,
      NULL AS permission_type,
      NULL AS permission_application_name,
      NULL::bigint AS permission_risk,
      NULL AS permission_root,
      NULL AS permission_policy_violated,
      NULL AS permission_entitlements,
      brole.brole_id,
      brole.brole_name,
      brole.description AS brole_description,
      brole.risk AS brole_risk,
      brole.update_time AS brole_update_time,
      NULL AS supervisee_id,
      NULL AS supervisee_display_name,
      NULL AS supervisee_email,
      NULL AS supervisee_department,
      NULL AS supervisee_job_title,
      NULL::bigint AS trole_policy_id,
      NULL AS trole_item_name,
      NULL AS trole_unique_policy_id,
      NULL AS trole_description,
      NULL::bigint AS trole_risk,
      NULL AS trole_policy_violated,
      NULL AS account_name,
      NULL AS account_id,
      NULL::bigint AS account_risk,
      NULL AS account_description,
      NULL AS account_application_name
    FROM suser_brole_v brole
      RIGHT JOIN suser_sod_case_info_v users ON users.unique_user_id = brole.user_id
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN supervisee.unique_user_id IS NOT NULL THEN '4-SUPERVISEE' 
          ELSE '4-NO-SUPERVISEE'
        END AS detail_type,
      NULL::character varying AS group_id,
      NULL AS group_name,
      NULL AS group_description,
      NULL::bigint AS permission_id,
      NULL AS permission_name,
      NULL AS permission_description,
      NULL AS permission_type,
      NULL AS permission_application_name,
      NULL::bigint AS permission_risk,
      NULL AS permission_root,
      NULL AS permission_policy_violated,
      NULL AS permission_entitlements,
      NULL::bigint AS brole_id,
      NULL AS brole_name,
      NULL AS brole_description,
      NULL::bigint AS brole_risk,
      NULL::timestamp with time zone AS brole_update_time,
      supervisee.unique_user_id AS supervisee_id,
      supervisee.display_name AS supervisee_display_name,
      supervisee_email.email  supervisee_email,
      supervisee.department AS supervisee_department,
      supervisee.job_title AS supervisee_job_title,
      NULL::bigint AS trole_policy_id,
      NULL AS trole_item_name,
      NULL AS trole_unique_policy_id,
      NULL AS trole_description,
      NULL::bigint AS trole_risk,
      NULL AS trole_policy_violated,
      NULL AS account_name,
      NULL AS account_id,
      NULL::bigint AS account_risk,
      NULL AS account_description,
      NULL AS account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN suser supervisee ON supervisee.supervisor_unique_id = users.unique_user_id AND supervisee.deleted = '0' AND supervisee.effective = '1'
    LEFT JOIN 
        (SELECT
            suser.id suser,
            suser.display_name,
            string_agg(suser_email.email, ' | ' order by suser_email.email) As email
        FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
        GROUP BY suser.id, suser.display_name) supervisee_email ON supervisee.id = supervisee_email.suser
    UNION ALL
    SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN policy.id IS NOT NULL THEN '5-TROLE'
          ELSE '5-NO-TROLE'
        END AS detail_type,
      NULL::character varying AS group_id,
      NULL::character varying AS group_name,
      NULL::character varying AS group_description,
      NULL::bigint AS permission_id,
      NULL AS permission_name,
      NULL AS permission_description,
      NULL AS permission_type,
      NULL AS permission_application_name,
      NULL::bigint AS permission_risk,
      NULL AS permission_root,
      NULL AS permission_policy_violated,
      NULL AS permission_entitlements,
      NULL::bigint AS brole_id,
      NULL AS brole_name,
      NULL AS brole_description,
      NULL::bigint AS brole_risk,
      NULL::timestamp with time zone AS brole_update_time,
      NULL AS supervisee_id,
      NULL AS supervisee_display_name,
      NULL AS supervisee_email,
      NULL AS supervisee_department,
      NULL AS supervisee_job_title,
      policy.id AS trole_policy_id,
      policy.policy_name AS trole_item_name,
      policy.unique_policy_id AS trole_unique_policy_id,
      policy.policy_description AS trole_description,
      policy.policy_risk AS trole_risk,
      sod.policy_name AS trole_policy_violated,
      NULL AS account_name,
      NULL AS account_id,
      NULL::bigint AS account_risk,
      NULL AS account_description,
      NULL AS account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN role_perm_policy_v policy ON users.unique_user_id = policy.permission_user_id
      LEFT JOIN sod_case_violation_v sod ON sod.unique_id = policy.unique_policy_id AND users.id = sod.id
    UNION ALL
    SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN sac.account_id IS NOT NULL THEN '6-ACCOUNT'
          ELSE '6-NO-ACCOUNT'
        END AS detail_type,
      NULL::character varying AS group_id,
      NULL::character varying AS group_name,
      NULL::character varying AS group_description,
      NULL::bigint AS permission_id,
      NULL AS permission_name,
      NULL AS permission_description,
      NULL AS permission_type,
      NULL AS permission_application_name,
      NULL::bigint AS permission_risk,
      NULL AS permission_root,
      NULL AS permission_policy_violated,
      NULL AS permission_entitlements,
      NULL::bigint AS brole_id,
      NULL AS brole_name,
      NULL AS brole_description,
      NULL::bigint AS brole_risk,
      NULL::timestamp with time zone AS brole_update_time,
      NULL AS supervisee_id,
      NULL AS supervisee_display_name,
      NULL AS supervisee_email,
      NULL AS supervisee_department,
      NULL AS supervisee_job_title,
      NULL::bigint AS trole_policy_id,
      NULL AS trole_item_name,
      NULL AS trole_unique_policy_id,
      NULL AS trole_description,
      NULL::bigint AS trole_risk,
      NULL AS trole_policy_violated,
      sac.name AS account_name,
      sac.account_id,
      sac.risk AS account_risk,
      sac.description AS account_description,
      source.name AS account_application_name
    FROM suser_account_permission sap
      JOIN saccount sac ON sap.saccount = sac.id AND sac.effective = '1' AND sac.deleted = '0' AND sac.virtual = '0'
      JOIN data_source source ON sac.application = source.id
      RIGHT JOIN suser_sod_case_info_v users ON sap.suser = users.id
 )
  SELECT 
    current_user_access.id,
    current_user_access.display_name,
    current_user_access.job_title,
    current_user_access.email,
    current_user_access.department,
    current_user_access.user_id AS user_cn,
    current_user_access.first_name,
    current_user_access.last_name,
    current_user_access.risk,
    current_user_access.unique_user_id,
    current_user_access.supervisor,
    current_user_access.violations,
    current_user_access.detail_type,
    current_user_access.group_id,
    current_user_access.group_name,
    current_user_access.group_description,
    current_user_access.permission_id,
    current_user_access.permission_name,
    current_user_access.permission_description,
    current_user_access.permission_type,
    current_user_access.permission_application_name,
    current_user_access.permission_risk,
    current_user_access.permission_root,
    current_user_access.permission_policy_violated,
    current_user_access.permission_entitlements,
    current_user_access.brole_id,
    current_user_access.brole_name,
    current_user_access.brole_description,
    current_user_access.brole_risk,
    current_user_access.brole_update_time,
    current_user_access.supervisee_id,
    current_user_access.supervisee_display_name,
    current_user_access.supervisee_email,
    current_user_access.supervisee_department,
    current_user_access.supervisee_job_title,
    current_user_access.trole_policy_id,
    current_user_access.trole_item_name,
    current_user_access.trole_unique_policy_id,
    current_user_access.trole_description,
    current_user_access.trole_risk,
    current_user_access.trole_policy_violated,
    current_user_access.account_name,
    current_user_access.account_id,
    current_user_access.account_risk,
    current_user_access.account_description,
    current_user_access.account_application_name
  FROM current_user_access;

CREATE VIEW risk_level_v AS 
 SELECT
        CASE
            WHEN ism_global_config.key_col::text ~~ '%10%'::text THEN "substring"(ism_global_config.key_col::text, "position"(ism_global_config.key_col::text, 'riskLevel.'::text) + 10, 2)
            ELSE "substring"(ism_global_config.key_col::text, "position"(ism_global_config.key_col::text, 'riskLevel.'::text) + 10, 1)
        END AS risk_level,
        CASE
            WHEN ism_global_config.key_col::text ~~ '%Label%'::text THEN replace("substring"(ism_global_config.key_col::text, "position"(ism_global_config.key_col::text, 'Label.'::text) + 6), '-'::text, '_'::text)
            ELSE NULL::text
        END AS locale,
        CASE
            WHEN ism_global_config.key_col::text ~~ '%Label%'::text THEN 'Label'::text
            WHEN ism_global_config.key_col::text ~~ '%Max'::text THEN 'Max'::text
            WHEN ism_global_config.key_col::text ~~ '%Min'::text THEN 'Min'::text
            ELSE NULL::text
        END AS key_col,
    ism_global_config.val_col
   FROM ism_global_config
  WHERE ism_global_config.key_col::text ~~ 'com.netiq.iac.persistence.datacuration.riskLevel.%'::text;

CREATE VIEW spermission_v AS
SELECT spermission.id,
    spermission.account_grant,
    spermission.assignable,
    spermission.authoritative,
    spermission.bound_by_id,
    spermission.boundable,
    spermission.cost,
    spermission.deleted,
    spermission.description,
    spermission.display_name,
    spermission.driver_id,
    spermission.effective,
    spermission.exclude_from_catalog,
    spermission.is_curated,
    spermission.logical_id,
    spermission.merged,
    spermission.name,
    spermission.not_reviewable,
    spermission.permission_id,
    spermission.prov_attribute,
    spermission.prov_by_unique_permission_id,
    spermission.prov_native_value,
    spermission.prov_unique_driver_id,
    spermission.provisionable,
    spermission.risk,
    spermission.type,
    spermission.unique_bound_by_id,
    spermission.unique_permission_id,
    spermission.value,
    spermission.value_type,
    spermission.application,
    spermission.base_application,
    spermission.collected_entity,
    spermission.data_collection,
    spermission.effective_entity,
    spermission.sversion,
    spermission.auto_generated,
    spermission.prov_unique_permission_id,
    perm_owner.perm_owner_name
   FROM spermission
     LEFT JOIN (SELECT rso.permission,
            string_agg(suser.display_name::text, ' | '::text) AS perm_owner_name
           FROM (SELECT DISTINCT owner_unique_id,
        permission
        FROM resolved_spermission_owner) rso
             JOIN suser ON suser.unique_user_id::text = rso.owner_unique_id::text AND suser.deleted = false AND suser.effective = true
          GROUP BY rso.permission) perm_owner ON perm_owner.permission = spermission.id
  WHERE (spermission.exclude_from_catalog = false OR spermission.exclude_from_catalog IS NULL) AND spermission.effective = true;


ALTER TABLE spermission_v
  OWNER TO igops;
GRANT ALL ON TABLE spermission_v TO igops;
GRANT SELECT ON TABLE spermission_v TO ig_report_role;

ALTER TABLE risk_level_v
  OWNER TO igops;
GRANT ALL ON TABLE risk_level_v TO igops;
GRANT SELECT ON TABLE risk_level_v TO ig_report_role;

ALTER TABLE current_user_access_v
  OWNER TO igops;
GRANT ALL ON TABLE current_user_access_v TO igops;
GRANT SELECT ON TABLE current_user_access_v TO ig_report_role;

ALTER TABLE suser_brole_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_brole_v TO igops;
GRANT SELECT ON TABLE suser_brole_v TO ig_report_role;

ALTER TABLE deleted_reviewers_v
  OWNER TO igops;
GRANT ALL ON TABLE deleted_reviewers_v TO igops;
GRANT SELECT ON TABLE deleted_reviewers_v TO ig_report_role;

ALTER TABLE sgroup_indirect_suser_v
  OWNER TO igops;
GRANT ALL ON TABLE sgroup_indirect_suser_v TO igops;
GRANT SELECT ON TABLE sgroup_indirect_suser_v TO ig_report_role;

ALTER TABLE sgroup_path_v
  OWNER TO igops;
GRANT ALL ON TABLE sgroup_path_v TO igops;
GRANT SELECT ON TABLE sgroup_path_v TO ig_report_role;

ALTER TABLE sgroups_subgroups_v
  OWNER TO igops;
GRANT ALL ON TABLE sgroups_subgroups_v TO igops;
GRANT SELECT ON TABLE sgroups_subgroups_v TO ig_report_role;

ALTER TABLE suser_sod_case_info_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_sod_case_info_v TO igops;
GRANT SELECT ON TABLE suser_sod_case_info_v TO ig_report_role;

ALTER TABLE sod_case_info_v
  OWNER TO igops;
GRANT ALL ON TABLE sod_case_info_v TO igops;
GRANT SELECT ON TABLE sod_case_info_v TO ig_report_role;

ALTER TABLE role_perm_policy_v
  OWNER TO igops;
GRANT ALL ON TABLE role_perm_policy_v TO igops;
GRANT SELECT ON TABLE role_perm_policy_v TO ig_report_role;

ALTER TABLE sod_policy_all_v
  OWNER TO igops;
GRANT ALL ON TABLE sod_policy_all_v TO igops;
GRANT SELECT ON TABLE sod_policy_all_v TO ig_report_role;

ALTER TABLE sod_policy_v
  OWNER TO igops;
GRANT ALL ON TABLE sod_policy_v TO igops;
GRANT SELECT ON TABLE sod_policy_v TO ig_report_role;

ALTER TABLE policy_owners_v
  OWNER TO igops;
GRANT ALL ON TABLE policy_owners_v TO igops;
GRANT SELECT ON TABLE policy_owners_v TO ig_report_role;

ALTER TABLE sod_case_violation_v
  OWNER TO igops;
GRANT ALL ON TABLE sod_case_violation_v TO igops;
GRANT SELECT ON TABLE sod_case_violation_v TO ig_report_role;

ALTER TABLE sod_policy_violation_v
  OWNER TO igops;
GRANT ALL ON TABLE sod_policy_violation_v TO igops;
GRANT SELECT ON TABLE sod_policy_violation_v TO ig_report_role;

ALTER TABLE spermission_associations_v
  OWNER TO igops;
GRANT ALL ON TABLE spermission_associations_v TO igops;
GRANT SELECT ON TABLE spermission_associations_v TO ig_report_role;

ALTER TABLE spermission_suser_v
  OWNER TO igops;
GRANT ALL ON TABLE spermission_suser_v TO igops;
GRANT SELECT ON TABLE spermission_suser_v TO ig_report_role;

ALTER TABLE business_role_nested_v
  OWNER TO igops;
GRANT ALL ON TABLE business_role_nested_v TO igops;
GRANT SELECT ON TABLE business_role_nested_v TO ig_report_role;

ALTER TABLE admin_auth_assignments_v 
  OWNER TO igops;
GRANT ALL ON TABLE admin_auth_assignments_v  TO igops;
GRANT SELECT ON TABLE admin_auth_assignments_v  TO ig_report_role;